/*
 *  Copyright (C) 1999-2001 Harri Porten (porten@kde.org)
 *  Copyright (C) 2003, 2004, 2005, 2006, 2008 Apple Inc. All rights reserved.
 *  Copyright (C) 2007 Samuel Weinig <sam@webkit.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef GdomDOMBinding_h
#define GdomDOMBinding_h

#include <kjs/interpreter.h>
#include <kjs/lookup.h>
#include <wtf/Noncopyable.h>

#include "gdomNode.h"
#include "GDOMObject.h"
#include "intgdomNode.h"

namespace WebCore {

    class AtomicString;
    class Document;
    class Event;
    class Frame;
    class KURL;
    class Node;
    class String;

#if ENABLE(SVG)
    class SVGElement;
#endif

    class GDOMScriptInterpreter {
    public:
        static void* getDOMObject(void* objectHandle);
        static void* putDOMObject(void* objectHandle, GDOMObject*);
        static void forgetDOMObject(void* objectHandle);

        static void* getDOMNodeForDocument(Document*, Node*);
        static void* putDOMNodeForDocument(Document*, Node*, GDOMNode* nodeWrapper);
        static void forgetDOMNodeForDocument(Document*, Node*);
        static void forgetAllDOMNodesForDocument(Document*);
		#if 0
        static void updateDOMNodeDocument(Node*, Document* oldDoc, Document* newDoc);
        static void markDOMNodesForDocument(Document*);
		#endif
    };

    // Retrieve from cache, or create, a Gdom wrapper for a DOM object.
    template<class DOMObj, class GdomDOMObj> inline void* cacheDOMObject(DOMObj* domObj)
    {
        if (!domObj)
            return NULL; /* XXX hmmm... */
        if (void* ret = GDOMScriptInterpreter::getDOMObject(domObj))
            return ret;
        GDOMObject* ret = new GdomDOMObj(domObj);
        return GDOMScriptInterpreter::putDOMObject(domObj, ret);
    }

#if ENABLE(SVG)
    // Retrieve from cache, or create, a Gdom wrapper for an SVG DOM object.
    template<class DOMObj, class GdomDOMObj> inline void* cacheSVGDOMObject(DOMObj* domObj, SVGElement* context)
    {
        if (!domObj)
            return NULL; /* XXX hmmm... */
        if (void* ret = GDOMScriptInterpreter::getDOMObject(domObj))
            return ret;
        GDOMObject* ret = new GdomDOMObj(domObj, context);
        return GDOMScriptInterpreter::putDOMObject(domObj, ret);
    }
#endif

} // namespace WebCore

#endif // GdomDOMBinding_h
